package com.jln.common.utils;

import com.jln.common.constant.LocalDateTimeConst;
import org.apache.commons.lang3.time.DateFormatUtils;

import java.text.DecimalFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.*;
import java.time.format.DateTimeFormatter;
import java.time.temporal.ChronoUnit;
import java.util.*;

/**
 * 日期工具类, 继承org.apache.commons.lang.time.DateUtils类
 *
 * @author zdc
 * @version 2020-04-23
 */
public class DateUtils extends org.apache.commons.lang3.time.DateUtils {

	private static String[] parsePatterns = {
			LocalDateTimeConst.DATE_PATTERN,
			LocalDateTimeConst.DATE_TIME_PATTERN,
			LocalDateTimeConst.DATE_SHORTTIME_PATTERN,
			LocalDateTimeConst.DATE_YEAR_MONTH_PATTERN,
			"yyyy/MM/dd",
			"yyyy/MM/dd HH:mm:ss",
			"yyyy/MM/dd HH:mm",
			"yyyy/MM",
			"yyyy.MM.dd",
			"yyyy.MM.dd HH:mm:ss",
			"yyyy.MM.dd HH:mm",
			"yyyy.MM"
	};

	/**
	 * 得到当前日期字符串 格式（yyyy-MM-dd）
	 */
	public static String getDate() {
		return getDate(LocalDateTimeConst.DATE_PATTERN);
	}

	/**
	 * 得到当前日期字符串 格式（yyyy-MM-dd） pattern可以为：LocalDateTimeConst.DATE_PATTERN "HH:mm:ss" "E"
	 */
	public static String getDate(String pattern) {
		return DateFormatUtils.format(new Date(), pattern);
	}

	/**
	 * 获取若干天以前的日期（时间）
	 *
	 * @param days
	 * @param pattern
	 */
	public static String getSeveralDaysAgoDate(int days, String pattern) {
		Date date = new Date();
		Calendar calendar = Calendar.getInstance();
		calendar.setTime(date);
		calendar.add(calendar.DATE, -days);
		date = calendar.getTime();
		SimpleDateFormat format = new SimpleDateFormat(pattern);
		return format.format(date);
	}

	/**
	 * 得到日期字符串 默认格式（yyyy-MM-dd） pattern可以为："yyyy-MM-dd" "HH:mm:ss" "E"
	 */
	public static String formatDate(Date date, Object... pattern) {
		String formatDate = null;
		if (pattern != null && pattern.length > 0) {
			formatDate = DateFormatUtils.format(date, pattern[0].toString());
		} else {
			formatDate = DateFormatUtils.format(date, LocalDateTimeConst.DATE_PATTERN);
		}
		return formatDate;
	}

	/**
	 * 得到日期时间字符串，转换格式（yyyy-MM-dd HH:mm:ss）
	 */
	public static String formatDateTime(Date date) {
		return formatDate(date, LocalDateTimeConst.DATE_TIME_PATTERN);
	}

	/**
	 * 得到当前时间字符串 格式（HH:mm:ss）
	 */
	public static String getTime() {
		return formatDate(new Date(), LocalDateTimeConst.DEFAULT_TIME_FORMAT);
	}

	/**
	 * 得到当前日期和时间字符串 格式（yyyy-MM-dd HH:mm:ss）
	 */
	public static String getDateTime() {
		return formatDate(new Date(), LocalDateTimeConst.DATE_TIME_PATTERN);
	}

	/**
	 * 得到当前年份字符串 格式（yyyy）
	 */
	public static String getYear() {
		return formatDate(new Date(), "yyyy");
	}

	/**
	 * 得到当前月份字符串 格式（MM）
	 */
	public static String getMonth() {
		return formatDate(new Date(), "MM");
	}

	/**
	 * 得到当天字符串 格式（dd）
	 */
	public static String getDay() {
		return formatDate(new Date(), "dd");
	}

	/**
	 * 得到当前星期字符串 格式（E）星期几
	 */
	public static String getWeek() {
		return formatDate(new Date(), "E");
	}

	/**
	 * 日期型字符串转化为日期 格式
	 * { "yyyy-MM-dd", LocalDateTimeConst.DATE_TIME_PATTERN, "yyyy-MM-dd HH:mm",
	 * "yyyy/MM/dd", "yyyy/MM/dd HH:mm:ss", "yyyy/MM/dd HH:mm",
	 * "yyyy.MM.dd", "yyyy.MM.dd HH:mm:ss", "yyyy.MM.dd HH:mm" }
	 */
	public static Date parseDate(Object str) {
		if (str == null) {
			return null;
		}
		try {
			return parseDate(str.toString(), parsePatterns);
		} catch (ParseException e) {
			return null;
		}
	}

	/**
	 * 获取过去的天数
	 *
	 * @param date
	 * @return
	 */
	public static long pastDays(Date date) {
		long t = new Date().getTime() - date.getTime();
		return t / (24 * 60 * 60 * 1000);
	}

	/**
	 * 获取过去的小时
	 *
	 * @param date
	 * @return
	 */
	public static long pastHour(Date date) {
		long t = new Date().getTime() - date.getTime();
		return t / (60 * 60 * 1000);
	}

	/**
	 * 获取过去的分钟
	 *
	 * @param date
	 * @return
	 */
	public static long pastMinutes(Date date) {
		long t = new Date().getTime() - date.getTime();
		return t / (60 * 1000);
	}

	/**
	 * 转换为时间（天,时:分:秒.毫秒）
	 *
	 * @param timeMillis
	 * @return
	 */
	public static String formatDateTime(long timeMillis) {
		long day = timeMillis / (24 * 60 * 60 * 1000);
		long hour = (timeMillis / (60 * 60 * 1000) - day * 24);
		long min = ((timeMillis / (60 * 1000)) - day * 24 * 60 - hour * 60);
		long s = (timeMillis / 1000 - day * 24 * 60 * 60 - hour * 60 * 60 - min * 60);
		long sss = (timeMillis - day * 24 * 60 * 60 * 1000 - hour * 60 * 60 * 1000 - min * 60 * 1000 - s * 1000);
		return (day > 0 ? day + "," : "") + hour + ":" + min + ":" + s + "." + sss;
	}

	/**
	 * 获取两个日期之间的天数
	 *
	 * @param before
	 * @param after
	 * @return
	 */
	public static double getDistanceOfTwoDate(Date before, Date after) {
		long beforeTime = before.getTime();
		long afterTime = after.getTime();
		return (afterTime - beforeTime) / (1000 * 60 * 60 * 24);
	}

	//————————————————————————————————————————————

	/**
	 * 获取若干天后的日期LocalDate类型
	 *
	 * @param localDate
	 * @param days
	 * @return
	 */
	public static LocalDate getNextNDay(LocalDate localDate, long days) {
		return localDate.with(temporal -> temporal.plus(days, ChronoUnit.DAYS));
	}

	/**
	 * 获得LocalDate日期范围内天数
	 *
	 * @param startDate
	 * @param endDate
	 * @return
	 */
	public static long until(LocalDate startDate, LocalDate endDate) {
		return startDate.until(endDate, ChronoUnit.DAYS);
	}

	/**
	 * 得到时间范围内日期（根据周系数）
	 *
	 * @param dateFrom 开始日期
	 * @param dateEnd  结束日期
	 * @param weekDays 周系数
	 * @param zqjg     周期间隔
	 * @return
	 */
	public static List<LocalDate> getWeekDates(String dateFrom, String dateEnd, String weekDays, String zqjg) {
		Integer tag = 0;
		long time = 1l;
		long perDayMilSec = 24 * 60 * 60 * 1000;
		List<LocalDate> dateList = new ArrayList<LocalDate>();
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
		//需要查询的星期系数
		String strWeekNumber = weekForNum(weekDays);
		try {
			dateFrom = sdf.format(sdf.parse(dateFrom).getTime() - perDayMilSec);
			while (true) {
				time = sdf.parse(dateFrom).getTime();
				time = time + perDayMilSec;
				Date date = new Date(time);
				dateFrom = sdf.format(date);
				if (dateFrom.compareTo(dateEnd) <= 0) {
					//查询的某一时间的星期系数
					Integer weekDay = dayForWeek(date);
					Integer month = dayForMonth(date);
					//判断当期日期的星期系数是否是需要查询的
					if (strWeekNumber.indexOf(weekDay.toString()) != -1 && tag % Integer.parseInt(zqjg) == 0) {
						//System.out.println(dateFrom);
						dateList.add(LocalDate.parse(dateFrom, DateTimeFormatter.ofPattern("yyyy-MM-dd")));
					}
					if (weekDay == 1) {
						tag++;
					}
				} else {
					break;
				}
			}
		} catch (ParseException e1) {
			e1.printStackTrace();
		}
		return dateList;
	}

	/**
	 * 得到时间范围内日期（根据月系数）
	 *
	 * @param dateFrom
	 * @param dateEnd
	 * @param monthDays
	 * @param zqjg
	 * @return
	 */
	public static List<LocalDate> getMonthDates(String dateFrom, String dateEnd, String monthDays, String zqjg) {
		Integer tag = 0;
		long time = 1l;
		long perDayMilSec = 24 * 60 * 60 * 1000;
		List<LocalDate> dateList = new ArrayList<>();
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
		DateTimeFormatter fmt = DateTimeFormatter.ofPattern("yyyy-MM-dd");
		//需要查询的星期系数
		monthDays = monthDays.replace("月末", "32");
		Integer[] MonthNumber = monthForNum(monthDays);
		try {
			dateFrom = sdf.format(sdf.parse(dateFrom).getTime() - perDayMilSec);
			while (true) {
				time = sdf.parse(dateFrom).getTime();
				time = time + perDayMilSec;
				Date date = new Date(time);
				dateFrom = sdf.format(date);
				if (dateFrom.compareTo(dateEnd) <= 0) {
					//查询的某一时间的星期系数
					Integer monthDay = dayForMonth(date);
					Date maxDayForMonth = maxDayForMonth(date);
					//判断当期日期的星期系数是否是需要查询的
					boolean isContains = Arrays.asList(MonthNumber).contains(monthDay);
					LocalDate maxdate = date2LocalDate(maxDayForMonth);

					if (isContains && tag % Integer.parseInt(zqjg) == 0) {//
						dateList.add(LocalDate.parse(dateFrom, DateTimeFormatter.ofPattern("yyyy-MM-dd")));
					}
					boolean b = dateList.contains(maxdate);
					boolean a = monthDays.contains("32");
					if (a && !b && LocalDate.parse(dateFrom, fmt).equals(maxdate) && tag % Integer.parseInt(zqjg) == 0) {
						dateList.add(date2LocalDate(maxDayForMonth));
					}
					if (maxDayForMonth.compareTo(date) == 0) {
						tag++;
					}
				} else {
					break;
				}
			}
		} catch (ParseException e1) {
			e1.printStackTrace();
		}
		return dateList;
	}

	/**
	 * 得到当期时间的周系数。星期日：1，星期一：2，星期二：3，星期三：4，星期四：5，星期五：6，星期六：7
	 *
	 * @param date
	 * @return
	 */
	public static Integer dayForWeek(Date date) {
		Calendar calendar = Calendar.getInstance();
		calendar.setTime(date);
		return calendar.get(Calendar.DAY_OF_WEEK);
	}

	/**
	 * 得到对应星期的系数  星期日：1，星期一：2，星期二：3，星期三：4，星期四：5，星期五：6，星期六：7
	 *
	 * @param weekDays 星期格式  星期一|星期二
	 */
	public static String weekForNum(String weekDays) {
		//返回结果为组合的星期系数
		String weekNumber = "";
		//解析传入的星期
		if (weekDays.indexOf("|") != -1) {//多个星期数
			String[] strWeeks = weekDays.split("\\|");
			for (int i = 0; i < strWeeks.length; i++) {
				weekNumber = weekNumber + "" + getWeekNum(strWeeks[i]).toString();
			}
		} else {//一个星期数
			weekNumber = getWeekNum(weekDays).toString();
		}
		return weekNumber;
	}

	/**
	 * 将星期转换为对应的系数  星期日：1，星期一：2，星期二：3，星期三：4，星期四：5，星期五：6，星期六：7
	 *
	 * @param strWeek
	 * @return
	 */
	public static Integer getWeekNum(String strWeek) {
		Integer number = 1;//默认为星期日
		if ("星期日".equals(strWeek)) {
			number = 1;
		} else if ("星期一".equals(strWeek)) {
			number = 2;
		} else if ("星期二".equals(strWeek)) {
			number = 3;
		} else if ("星期三".equals(strWeek)) {
			number = 4;
		} else if ("星期四".equals(strWeek)) {
			number = 5;
		} else if ("星期五".equals(strWeek)) {
			number = 6;
		} else if ("星期六".equals(strWeek)) {
			number = 7;
		} else {
			number = 1;
		}
		return number;
	}

	/**
	 * Date 转 LocalDate
	 *
	 * @param date
	 * @return
	 */
	public static LocalDate date2LocalDate(Date date) {
		Instant instant = date.toInstant();
		ZonedDateTime zdt = instant.atZone(ZoneId.systemDefault());
		LocalDate localDate = zdt.toLocalDate();
		return localDate;
	}

	/**
	 * 等到当期时间的天系数。2020-04-01  -->   1
	 *
	 * @param date
	 * @return
	 */
	public static Integer dayForMonth(Date date) {
		Calendar calendar = Calendar.getInstance();
		calendar.setTime(date);
		return calendar.get(Calendar.DAY_OF_MONTH);
	}

	/**
	 * 获取当前日期的最大一天
	 *
	 * @param date
	 * @return
	 */
	public static Date maxDayForMonth(Date date) {
		Calendar calendar = Calendar.getInstance();
		calendar.setTime(date);
		calendar.set(Calendar.DAY_OF_MONTH, 1);// 设定当前时间为每月一号
		calendar.roll(Calendar.DAY_OF_MONTH, -1);
		return calendar.getTime();
	}

	/**
	 * 得到对应月份的系数
	 */
	public static Integer[] monthForNum(String monthDays) {
		HashMap<String, Integer> monthMap = new HashMap<>();
		for (int i = 1; i <= 31; i++) {
			monthMap.put(String.valueOf(i), i);
		}
		monthMap.put("月末", 32);

		Integer[] monthNumber = new Integer[32];
		//解析传入的星期
		if (monthDays.indexOf("|") != -1) {//多个星期数
			String[] strMonths = monthDays.split("\\|");
			for (int i = 0; i < strMonths.length; i++) {
				monthNumber[i] = Integer.parseInt(strMonths[i]);
			}
		} else {//一个星期数
			monthNumber[0] = Integer.parseInt(monthDays);
		}
		return monthNumber;
	}

	/**
	 * @param args
	 * @throws ParseException
	 */
	public static void main(String[] args) throws ParseException {
//		System.out.println(formatDate(parseDate("2010/3/6")));
//		System.out.println(getDate("yyyy年MM月dd日 E"));
//		long time = new Date().getTime()-parseDate("2012-11-19").getTime();
//		System.out.println(time/(24*60*60*1000));
		List<String> between = getMonthBetween("2020-02-26 00:00:00", "2020-02-28 00:00:00");
		System.out.println(JsonUtils.toJson(between));

	}

	/**
	 * 获取一段时间里面有几个月份
	 *
	 * @param startDate 开始时间
	 * @param endDate   结束时间
	 * @return
	 */
	public static List<String> getMonthBetween(String startDate, String endDate) {
		ArrayList<String> result = new ArrayList<String>();
		try {
			SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM");//格式化为年月

			Calendar min = Calendar.getInstance();
			Calendar max = Calendar.getInstance();
			min.setTime(sdf.parse(startDate));
			min.set(min.get(Calendar.YEAR), min.get(Calendar.MONTH), 1);

			max.setTime(sdf.parse(endDate));
			max.set(max.get(Calendar.YEAR), max.get(Calendar.MONTH), 2);

			Calendar curr = min;
			while (curr.before(max)) {
				result.add(sdf.format(curr.getTime()));
				curr.add(Calendar.MONTH, 1);
			}
		} catch (ParseException e) {
			e.printStackTrace();
		}

		return result;
	}

	/**
	 * LocalDateTime 转字符串类型时间戳
	 *
	 * @param dateTime
	 * @return
	 */
	public static String getStringDate(LocalDateTime dateTime) {
		return DateTimeFormatter.ofPattern(LocalDateTimeConst.DATE_TIME_PATTERN).format(dateTime);
	}

	/****
	 * 传入具体日期 ，返回具体日期增加一个月。
	 * @param date 日期(2017-04-13)
	 * @return 2017-05-13
	 * @throws ParseException
	 */
	public static String subMonth(String date) throws ParseException {
		SimpleDateFormat sdf = new SimpleDateFormat(LocalDateTimeConst.DATE_TIME_PATTERN);
		Date dt = sdf.parse(date);
		Calendar rightNow = Calendar.getInstance();
		rightNow.setTime(dt);
		rightNow.add(Calendar.MONTH, 1);
		Date dt1 = rightNow.getTime();
		String reStr = sdf.format(dt1);
		return reStr;
	}

	/**
	 * 字符串转 LocalDateTime
	 *
	 * @param timeStr
	 * @return
	 */
	public static LocalDateTime stringToLocal(String timeStr) {
		DecimalFormat df = new DecimalFormat();
		df.applyPattern(LocalDateTimeConst.DATE_TIME_PATTERN);
		String format = df.format(timeStr);
		DateTimeFormatter timeDtf = DateTimeFormatter.ofPattern(LocalDateTimeConst.DATE_TIME_PATTERN);
		return LocalDateTime.parse(format, timeDtf);
	}

	public static LocalDateTime doubleToDate(Double dVal) {
		LocalDateTime localDateTime = LocalDateTime.now();
		Date tDate = new Date();
		//系统时区偏移 1900/1/1 到 1970/1/1 的 25569 天
		long localOffset = tDate.getTimezoneOffset() * 60000;
		tDate.setTime((long) ((dVal - 25569) * 24 * 3600 * 1000 + localOffset));
		Instant instant = tDate.toInstant();
		ZoneId zoneId = ZoneId.systemDefault();
		return instant.atZone(zoneId).toLocalDateTime();
	}


}
